
--[[
=head1 NAME

applets.Apps.AppsApplet - App starter

=head1 DESCRIPTION

This applets is used to start Apps.

=head1 FUNCTIONS

Applet related methods are described in L<jive.Applet>. 
AppsApplet overrides the following methods:

=cut
--]]


-- stuff we use
local setmetatable, tonumber, tostring = setmetatable, tonumber, tostring

local io                     = require("io")
local oo                     = require("loop.simple")
local string                 = require("string")
local lfs                    = require("lfs")

local Applet                 = require("jive.Applet")
local SimpleMenu             = require("jive.ui.SimpleMenu")
local Window                 = require("jive.ui.Window")
local Framework              = require("jive.ui.Framework")
local os                     = require("os")

local appletManager = appletManager
local EVENT_CONSUME = jive.ui.EVENT_CONSUME
local EVENT_QUIT    = jive.ui.EVENT_QUIT

module(...)
oo.class(_M, Applet)


function menu(self, menuItem)
	apps = {}
	for file in lfs.dir[[/usr/share/menu]] do
		if lfs.attributes("/usr/share/menu/"..file, "mode") == "file" then
			local fh = io.open("/usr/share/menu/"..file, "r")
			local menu = fh:read("*all")
			fh:close()
			local title = string.match(menu, 'title="([^"]+)"')
			local hide = string.match(menu, 'hide="true"')
			if title != nil and title != "SqueezePlay" and hide == nil then
				apps[#apps + 1] = { 
					text = title,
					callback = function(event, menuItem)
						-- Start the app
						appletManager:callService("disconnectPlayer")
						os.execute("setting call apps.start "..menuItem.text)
						Framework:quit()
						return (EVENT_CONSUME | EVENT_QUIT)
					end 
				}
			end
		end
	end

	local menu = SimpleMenu("menu", apps)
	local window = Window("window", self:string("START_AN_APP"))
	window:addWidget(menu)

	self:tieAndShowWindow(window)
	return window
end


--[[

=head1 LICENSE

This source code is public domain. It is intended for you to use as a starting
point to create your own applet.

=cut
--]]
